package com.xkrs.utils;

import org.apache.commons.codec.binary.Base64;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;

/**
 * @author XinYi Song
 */
public class FileUtil {
    /**
     * 	根据base64判断图片是否为tiff
     * @param base64
     * @return
     */
    public boolean checkImageBase64Format(String base64) {
        byte[] b=java.util.Base64.getDecoder().decode(base64);
        try {
            // 	判断是否为tiff格式
            if((b[0] & 0xFF) == 0x49 && (b[1] & 0xFF)==0x49 &&  (b[2] & 0xFF)==0x2A){
                return true;
            }else {
                return false;
            }
        }catch(Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /**
     * 	根据BASE64字符串生成图片文件，此方法生成tiff格式
     * @param base64
     * @param fileName
     * @param dictionary
     */
    public void base64ToFile(String base64,String fileName,String dictionary) {
        File file =null;
        File dir=new File(dictionary);
        //	无目录的情况下创建一个目录,会受权限影响,最好是已存在的目录
        if(!dir.exists() && dir.isDirectory()) {
            dir.mkdirs();
        }
        FileOutputStream fos=null;
        BufferedOutputStream bos=null;

        try {
            byte [] bytes=java.util.Base64.getDecoder().decode(base64);
            //目录+文件名作为输出文件的全路径
            file=new File(dictionary+fileName);
            fos= new FileOutputStream(file);
            bos=new BufferedOutputStream(fos);
            bos.write(bytes);
        }catch(Exception e) {
            e.printStackTrace();
        }finally {
            if(bos!=null) {
                try {
                    bos.close();
                }catch(IOException e) {
                    e.printStackTrace();
                }
            }
            if(fos!=null) {
                try {
                    fos.close();
                }catch(IOException e) {
                    e.printStackTrace();
                }
            }
        }


    }


    /**
     * 	将tiff图片转化为jpg，生成新的文件
     * @param oldPath	原图片的全路径
     * @param newPath	生成新的图片的全路径
     */
    public void tiffToJpg(String oldPath,String newPath) {
        try {
            BufferedImage bufferegImage= ImageIO.read(new File(oldPath));
            //可以是png等其它图片格式
            ImageIO.write(bufferegImage,"jpg",new File(newPath));

        }catch(IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 	将任意图片文件转为base64，读字节是最快的方式
     * @param filePath	图片文件的全路径
     * @return
     */
    public String imageToBase64(String filePath) {
        byte [] data =null;
        try {
            InputStream in =new FileInputStream(filePath);
            data=new byte[in.available()];
            in.read(data);
            in.close();
        }catch(Exception e) {
            e.printStackTrace();
            return null;
        }

        return new String(Base64.encodeBase64(data));
    }

    public static void main(String[] args) {
        String oldPath = "C:/Users/xkrs/Desktop/WF/分布/maize";
        String newPath = "E:/img";
        try {
            BufferedImage bufferegImage=ImageIO.read(new File(oldPath));
            //可以是png等其它图片格式
            ImageIO.write(bufferegImage,"jpg",new File(newPath));

        }catch(IOException e) {
            e.printStackTrace();
        }
    }
}
